# -*- coding: utf-8 -*-
import os
import logging
from PIL import Image

from django import template
from django.contrib.staticfiles import finders
from django.contrib.staticfiles.templatetags.staticfiles import static

from django_webp.utils import WEBP_STATIC_URL, WEBP_STATIC_ROOT, WEBP_DEBUG

register = template.Library()


def _get_static_image(image_url):
    return static(image_url)


def _generate_path(image_path):
    """ creates all folders necessary until reach the file's folder """
    current_path = ''
    image_path = list(image_path.split('/'))
    image_path.reverse()

    while image_path:
        current_path = os.path.join(current_path, image_path.pop())
        if os.path.isdir(current_path):
            continue

        if image_path:
            # if isn't the last one
            os.mkdir(current_path)


def _generate_webp_image(image_path, image_url):
    real_url = os.path.splitext(image_url)[0] + '.webp'
    generated_path = os.path.join(WEBP_STATIC_ROOT, real_url)
    real_url = WEBP_STATIC_URL + real_url

    if not os.path.isfile(generated_path):
        # generating the image
        image = Image.open(image_path)
        try:
            _generate_path(generated_path)
            image.save(generated_path, 'WEBP')
        except KeyError:
            logger = logging.getLogger(__name__)
            logger.warn('WEBP is not installed in pillow')
            return _get_static_image(image_url)
        except (IOError, OSError):
            logger = logging.getLogger(__name__)
            logger.warn('WEBP image could not be saved in %s' % real_url)
            return _get_static_image(image_url)

    return real_url


def _get_generated_image(image_url):
    """ Returns the url to the webp gerenated image,
        if the image doesn't exist or the generetion fails,
        it returns the regular static url for the image """

    image_path = finders.find(image_url)
    if not image_path:
        return _get_static_image(image_url)

    return _generate_webp_image(image_path, image_url)


@register.simple_tag(takes_context=True)
def webp(context, value, force_static=WEBP_DEBUG):
    supports_webp = context.get('supports_webp', False)
    if not supports_webp or force_static:
        return _get_static_image(value)
    else:
        return _get_generated_image(value)



# -*- coding: utf-8 -*-
import unittest

from django.http import HttpRequest

from django_webp.tests.test_main import USER_AGENTS
from django_webp.context_processors import webp


class ContextProcessorTest(unittest.TestCase):

    def setUp(self):
        self.request = HttpRequest()


    def test_common_return(self):
        """ Must return False in the dic """
        result = webp(self.request)
        supports_webp = result.get('supports_webp', None)
        self.assertFalse(supports_webp)


    def test_by_user_agent(self):
        """ Giving a valid user agent, should return True """
        self.request.META['HTTP_USER_AGENT'] = USER_AGENTS[0]
        result = webp(self.request)
        supports_webp = result.get('supports_webp', None)
        self.assertTrue(supports_webp)


    def test_by_http_accept_header(self):
        """
        Giving a valid http accept header, shold return True
        @see -> https://www.igvita.com/2013/05/01/deploying-webp-via-accept-content-negotiation/
        """
        self.request.META['HTTP_ACCEPT'] = 'image/webp'
        result = webp(self.request)
        supports_webp = result.get('supports_webp', None)
        self.assertTrue(supports_webp)


# -*- coding: utf-8 -*-
import os

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.contrib.staticfiles.templatetags.staticfiles import static


STATIC_ROOT = getattr(settings, 'STATIC_ROOT', '') or ''
WEBP_STATIC_ROOT = getattr(settings, 'WEBP_STATIC_ROOT', 'WEBP_CACHE')
WEBP_STATIC_ROOT = os.path.join(STATIC_ROOT, WEBP_STATIC_ROOT)

WEBP_STATIC_URL = static(getattr(settings, 'WEBP_STATIC_URL', 'WEBP_CACHE/'))

WEBP_DEBUG = getattr(settings, 'WEBP_DEBUG', settings.DEBUG)


if not WEBP_STATIC_URL.endswith('/'):
    raise ImproperlyConfigured("If set, WEBP_STATIC_URL must end with a slash")


# -*- coding: utf-8 -*-
import os
import unittest
from PIL import Image

from django.test.client import Client
from django.contrib.staticfiles import finders
from django.core.management import call_command
from django.core.management.base import CommandError

from django_webp.utils import WEBP_STATIC_ROOT


USER_AGENTS = [
    # Chrome
    'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.166 Safari/535.19',
    'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/34.0.1847.131 Safari/537.36',
    'Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/34.0.1847.116 Safari/537.36 Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B334b Safari/531.21.10',

    # Opera
    'Opera/9.80 (Windows NT 6.0) Presto/2.12.388 Version/12.14',
    'Mozilla/5.0 (Windows NT 6.0; rv:2.0) Gecko/20100101 Firefox/4.0 Opera 12.14',
    'Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8; U; fr) Presto/2.9.168 Version/11.52',
]

IMAGE_PNG_PATH = finders.find('django_webp/python.png')


class MainTest(unittest.TestCase):

    def test_pillow(self):
        """ Checks if current pillow installation
        has support to WEBP """
        image = Image.open(IMAGE_PNG_PATH)
        try:
            image.load()
            self.assertTrue(True)
        except:
            self.assertFail("There is no support for webp")


    def test_index(self):
        client = Client()
        response = client.get('/')
        
        self.assertEqual(response.status_code, 200)


    def test_clean_webp_images_command(self):
        call_command('collectstatic', interactive=False, verbose=False)
        self.assertRaises(CommandError, call_command, 'clean_webp_images')



# -*- coding: utf-8 -*-
import unittest
import os
import shutil

from django.template import Template, Context
from django.test.utils import override_settings
from django.contrib.staticfiles.templatetags.staticfiles import static

from django_webp.templatetags.webp import webp
from django_webp.utils import WEBP_STATIC_URL, WEBP_STATIC_ROOT


class TemplateTagTest(unittest.TestCase):

    def setUp(self):
        self.supported_url = WEBP_STATIC_URL + 'django_webp/python.webp'
        self.unsupported_url = static('django_webp/python.png')
        self.context = Context({})


    def tearDown(self):
        # cleaning the folder the files here
        try:
            shutil.rmtree(WEBP_STATIC_ROOT)
        except:
            pass


    def _assertFile(self, file_path, msg=''):
        file_exist = os.path.isfile(file_path)
        msg = msg or ('file doesnt exist: %s' % file_path)
        self.assertTrue(file_exist, msg)

    def _get_valid_context(self):
        return Context({'supports_webp': True})


    def _get_invalid_context(self):
        return Context({'supports_webp': False})


    def _render_template(self, html, context={}):
        return Template(html).render(Context(context))


    def test_unexistent_image(self):
        """ If the given image doesn't exist, return the static url """
        context = self._get_valid_context()
        result = webp(context, 'django_webp/this_image_doesnt_exist.gif')
        expected = static('django_webp/this_image_doesnt_exist.gif')
        self.assertEqual(result, expected)


    def test_supports_webp_false(self):
        context = self._get_invalid_context()
        result = webp(context, 'django_webp/python.png')
        expected = static('django_webp/python.png')
        self.assertEqual(result, expected)


    def test_templatetag(self):
        """ checks the returned url from the webp function """
        context = self._get_valid_context()
        result = webp(context, 'django_webp/python.png')
        self.assertEqual(self.supported_url, result)
        self._assertFile(result, 'file %s should have been created' % result)


    def test_templatetag_in_template(self):
        html = '{% load webp %}{% webp "django_webp/python.png" %}'
        rendered = self._render_template(html, context=self._get_valid_context())
        self.assertEqual(self.supported_url, rendered)
        self._assertFile(rendered, 'file %s should have been created' % rendered)


    def test_debug_true(self):
        """ if DEBUG = True, should always return the static url """
        context = self._get_valid_context()
        result = webp(context, 'django_webp/python.png', force_static=True)
        self.assertEqual(self.unsupported_url, result)



# -*- coding: utf-8 -*-

from django.conf.urls import patterns, url
from django.contrib.staticfiles.urls import staticfiles_urlpatterns

urlpatterns = patterns('django_webp.views',
    url(r'^$', 'index'),
) + staticfiles_urlpatterns()

# Only for test purposes



# -*- coding: utf-8 -*-
import httpagentparser

from django.conf import settings


WEBP_VALID_BROWSERS = getattr(settings, 'WEBP_VALID_BROWSERS', ['Chrome', 'Opera', 'Opera Mobile'])


def _check_by_user_agent(user_agent):
    """ Checks if the client accepts checking the given user_agent """
    if user_agent:
        data = httpagentparser.detect(user_agent)
        if 'browser' in data:
            return data['browser']['name'] in WEBP_VALID_BROWSERS

    return False


def _check_by_http_accept_header(http_accept):
    return 'webp' in http_accept


def webp(request):
    """ Adds `supports_webp` value in the context """
    user_agent = request.META.get('HTTP_USER_AGENT')
    http_accept = request.META.get('HTTP_ACCEPT', '')

    if _check_by_http_accept_header(http_accept):
        supports_webp = True
    elif _check_by_user_agent(user_agent):
        supports_webp = True
    else:
        supports_webp = False

    return {'supports_webp': supports_webp}



import shutil

from django.core.management.base import BaseCommand, CommandError

from django_webp.utils import WEBP_STATIC_ROOT

class Command(BaseCommand):
    help = 'Removes all cached webp images'


    def handle(self, *args, **options):
        try:
            shutil.rmtree(WEBP_STATIC_ROOT)
            self.stdout.write('Folder %s removed' % WEBP_STATIC_ROOT)
        except:
            raise CommandError('Folder %s was already removed' % WEBP_STATIC_ROOT)
